class CategoriesItemsBase extends HTMLElement {
  connectedCallback() {
    this.setAttributes();
    this.linkClickHandler = this.linkClickHandler.bind(this);
    this.renderComponent();
  }

  linkClickHandler(e) {
    e.preventDefault();
    const currentCategory = window.commercetools.getQueryParamByIndex(
      'category',
      this.componentConfig.productListIndex,
    );
    const paramName = window.commercetools.getParamNameByIndex(
      'category',
      this.componentConfig.productListIndex,
    );
    const category = new URL(e.target.href).searchParams.get(paramName);

    // Early exit if category stills the same.
    if (category === currentCategory) {
      return;
    }
    const url = new URL(window.location);
    this.removeIndexQueryParams(url, this.componentConfig.productListIndex);
    url.searchParams.set(paramName, category);

    const targetPage = this.componentConfig.targetPage.slice(1);

    if (targetPage.trim() !== '' && url.pathname !== Drupal.url(targetPage)) {
      url.pathname = Drupal.url(targetPage);
      window.location = url.toString();
    } else {
      window.history.pushState({ path: url.toString() }, '', url.toString());
    }

    window.commercetools.resetProductListResult(
      this.componentConfig.productListIndex,
    );
    const changeEvent = new CustomEvent('ct.filters.change', {
      detail: {
        productListIndex: this.componentConfig.productListIndex,
        newUrl: url,
      },
    });
    document.dispatchEvent(changeEvent);
  }

  // This method does not reference `this`, but is inherited classes use this method.
  // eslint-disable-next-line class-methods-use-this
  removeIndexQueryParams(url, index) {
    const paramsToRemove = window.commercetools.getRequestQueryParams(index);
    // eslint-disable-next-line no-restricted-syntax
    for (const [name, data] of Object.entries(paramsToRemove)) {
      url.searchParams.delete(data.originalKey);
    }
  }

  // eslint-disable-next-line class-methods-use-this
  setAttributes() {
    throw new Error("Method 'setAttributes()' must be implemented.");
  }

  // eslint-disable-next-line class-methods-use-this
  renderComponent() {
    throw new Error("Method 'renderComponent()' must be implemented.");
  }
}

window.commercetools.CategoriesItemsBase = CategoriesItemsBase;
